#include "control.h"

#include <iostream.h>
#include <GL/glut.h>
#include <math.h>
#include "lights.h"
#include "main.h"
#include "cloth.h"
#include "scene.h"
#include "camera.h"


enum {
	M_ADJUST_KS,
	M_ADJUST_KD,
	M_ADJUST_STIFFNESS,
	M_ADJUST_GRAVITY,
	M_ADJUST_WIND,

	M_RESET,
        M_HELP,
        M_QUIT
} MENU_ITEMS;


int mouse_state  = -1;
int mouse_button = -1;
int old_x = -1;
int old_y = -1;

float time     = 0.0;
float old_time = 0.0;
float dtime    = 0.0;
float kt       = 1.0;
float last     = 0.0;
int   frames   = 0;
float rate     = 0.0;

bool  visible   = true;
bool  framerate = true;
bool  paused    = false;


void pause ()
{
  paused = !paused;

  if (paused)
  {
  }
  else
  {
      time     = glutGet(GLUT_ELAPSED_TIME) / 1000.0 * kt;
      old_time = time;
      last     = time;
      frames   = 0;
  }

}


int make_menu ()
{
  int main = glutCreateMenu(menu_func);
  
  glutAddMenuEntry("Adjust ks", M_ADJUST_KS);
  glutAddMenuEntry("Adjust kd", M_ADJUST_KD);
  glutAddMenuEntry("Adjust stiffness", M_ADJUST_STIFFNESS);
  glutAddMenuEntry("Adjust gravity", M_ADJUST_GRAVITY);
  glutAddMenuEntry("Adjust wind", M_ADJUST_WIND);

  glutAddMenuEntry("Reset", M_RESET);
  glutAddMenuEntry("Help", M_HELP);
  glutAddMenuEntry("Quit", M_QUIT);

  glutAttachMenu(GLUT_RIGHT_BUTTON);

  return main;
}



void menu_func (int value)
{
  switch (value)
  {
  case M_ADJUST_KS:
      pause();
      cerr << "New spring coefficient [" << ks << "] : ";
      cin >> ks;
      pause();
      break;
      ;;

  case M_ADJUST_KD:
      pause();
      cerr << "New damped coefficient [" << kd << "] : ";
      cin >> kd;
      pause();
      break;
      ;;

  case M_ADJUST_STIFFNESS:
      pause();
      cerr << "New stiffness [" << stiffness << "] : ";
      cin >> stiffness;
      pause();
      break;
      ;;

  case M_ADJUST_GRAVITY:
      pause();
      cerr << "New gravity vector [ " << gravity[0] << ", " << gravity[1]
           << ", " << gravity[2] << " ] : ";
      cin >> gravity[0] >> gravity[1] >> gravity[2];
      pause();
      break;
      ;;

  case M_ADJUST_WIND:
      pause();
      cerr << "New wind vector [ " << windvec[0] << ", " << windvec[1]
           << ", " << windvec[2] << " ] : ";
      cin >> windvec[0] >> windvec[1] >> windvec[2];
      pause();
      break;
      ;;

  case M_RESET:
      initCloth();
      break;
      ;;

  case M_QUIT:  
      exit(0);
      break;
      ;;

  case M_HELP:
      menu_help();
      break;
      ;;

  default:
      break;
      ;;
  }
}


void keyboard_func (unsigned char key, int x, int y)
{
  switch (key)
  {
  case '1': case '2': case '3': case '4': case '5':
  case '6': case '7': case '8': case '9': case '0':
      key_set_time_dilation(key - 48);
      break;

  case 3:  // cntl-C
      exit(0);
      ;;

  case 'f':
      framerate = !framerate;
      if (framerate)
          cerr << "Showing framerate" << endl;
      else
          cerr << "Hiding framerate" << endl;
      break;
      ;;

  case 'a':
      anchor = !anchor;
      if (anchor)
          cerr << "Anchors attached" << endl;
      else
          cerr << "Anchors released" << endl;
      break;
      ;;

  case 's':
      sphere = !sphere;
      if (sphere)
          cerr << "Sphere on" << endl;
      else
          cerr << "Sphere off" << endl;
      break;
      ;;

  case 'i':
      internal = !internal;
      if (internal)
          cerr << "Internal collisions on" << endl;
      else
          cerr << "Internal collisions off" << endl;
      break;
      ;;

  case 'w':
      wind = !wind;
      if (wind)
          cerr << "Wind on" << endl;
      else
          cerr << "Wind off" << endl;
      break;
      ;;

  case 'p':
      pause();
      if (paused)
          cerr << "Paused" << endl; 
      else
          cerr << "Unpaused" << endl;
      break;
      ;;

  case 'v':
      if (camera_mode == C_STATIONARY)
      {
          camera_mode = C_TRACK;
          cerr << "Camera tracking cloth" << endl;
      }
      else
      {
          camera_mode = C_STATIONARY;
          cerr << "Camera stationary" << endl;
      }
      break;
      ;;

  default:
      cerr << "k " << (int) key << endl;;
      break;
      ;;
  }
}


void special_func (int key, int x, int y)
{
  switch (key)
  {
  default:
      cerr << "s " << (int) key << endl;;
      break;
      ;;
  }
}
  
        
void mouse_click_func (int button, int state, int x, int y)
{
  mouse_state  = state;
  mouse_button = button;
  old_x = x;
  old_y = y;
}
        
  
void mouse_move_func (int x, int y)
{
  if (mouse_button == GLUT_LEFT_BUTTON)
  {
      mouse_move_camera(x - old_x, y - old_y);
  }
  else if (mouse_button == GLUT_MIDDLE_BUTTON)
  {
      mouse_zoom_camera(x - old_x, y - old_y);
  }

  old_x = x;
  old_y = y;
}


void idle_func ()
{
  time  = glutGet(GLUT_ELAPSED_TIME) / 1000.0 * kt;
  dtime = time - old_time;

  if (framerate && (time - last > 5))
  {
      rate = frames / (time - last) / kt;
      cerr << "framerate: " << rate << " fps";
      if (rate <= 6.0)
          cerr << "\twarning - may become unstable!";
      cerr << endl;
      frames = 0;
      last   = time;
  }

  if (!paused)
      updateCloth();

  if (visible)
      glutPostRedisplay();
  old_time = time;
}


void visibility_func (int visibility)
{
  if (visibility == GLUT_VISIBLE)
      visible = true;
  else
      visible = false;
}


void menu_help ()
{
  cerr << endl
       << "key\t\taction" << endl
       << endl
       << "f\t\tToggle framerate" << endl
       << "a\t\tToggle anchors" << endl
       << "i\t\tToggle internal collisions" << endl
       << "s\t\tToggle sphere" << endl
       << "w\t\tToggle wind" << endl
       << "p\t\tPause simulation" << endl
       << "v\t\tToggle camera tracking" << endl
       << "0-9\t\tVary speed of simulation" << endl
       << endl;
}


void mouse_move_camera (int dx, int dy)
{
  camera_theta += dx * 0.5;
  camera_pitch += dy * 0.5;

  glutPostRedisplay();
}


void mouse_zoom_camera (int dx, int dy)
{
  camera_zoom += dy * 0.01;

  glutPostRedisplay();
}


void key_set_time_dilation (int k)
{
  pause();
  if (k == 0)
      k = 10;

  cerr << "speed: " << k << endl;

  kt = k * 0.2;
  pause();
}
